﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Data.MoveVectorData;
using HeuristicLab.Data.MoveVectorData.Moves;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;

namespace HeuristicLab.Encodings.MoveVectorEncoding.Manipulators
{
    [Item("SingleMoveManipulator", " Uniformly distributed change of a single position of a move.")]
    [StorableClass]
    public class SingleMoveManipulator : BoundedMoveVectorManipulator
    {

        [StorableConstructor]
        protected SingleMoveManipulator(bool deserializing) : base(deserializing) { }
        protected SingleMoveManipulator(SingleMoveManipulator original, Cloner cloner) : base(original, cloner) { }
        
        public SingleMoveManipulator() : base() { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new SingleMoveManipulator(this, cloner);
        }

        public static void Apply(IRandom random, MoveVector moves, IntMatrix bounds)
        {
            Manipulate(random, moves, bounds, random.Next(moves.Length));
        }

        public static void Manipulate(IRandom random, MoveVector moves, IntMatrix bounds, int index)
        {
            if (bounds == null || bounds.Rows == 0 || bounds.Columns < 2) throw new ArgumentException("SingleMoveManipulator: Invalid bounds specified", "bounds");
            int min = bounds[index % bounds.Rows, 0], max = bounds[index % bounds.Rows, 1], step = 1;
            if (bounds.Columns > 2) step = bounds[index % bounds.Rows, 2];

            moves[index].Randomize(random, min, max, step);
        }

        protected override void ManipulateBounded(IRandom random, MoveVector moves, IntMatrix bounds)
        {
            if (BoundsParameter.ActualValue == null) throw new InvalidOperationException("SingleMoveManipulator: Parameter " + BoundsParameter.ActualName + " could not be found.");
            Apply(random, moves, bounds);
        }
    }
}
